@@ -42,15 +42,17 @@ sub DESCRIPTION {
 
 ### CONSTANTS
 
-# Regexp compiled from dh_nodejs/dh_files.excluded
-my $fileIgnored =
-qr/^(?:a(?:ut(?:otest\.watchr|hors?)|ppveyor\.yml)|c(?:o(?:ntribute|pying)|hange(?:log|s?))|.*\.(?:m(?:arkdown|d)|pdf|txt)|r(?:ollup\.config\.js|eadme)|t(?:sconfig\.json|ests?\.js)|b(?:ench.*\.js|ower\.json)|g(?:runt|ulp)file\.js|(?:Docker|make)file|package-lock\.json|(?:license|\.).*|yarn\.lock|history)$/i;
+# Regexp compiled from dh_nodejs/dh_root_files.excluded
+my $rootFilesIgnored =
+qr/^(?:c(?:o(?:ntribute|pying)|hange(?:log|s?)|akefile)|a(?:ut(?:otest\.watchr|hors?)|ppveyor\.yml)|g(?:ulpfile(?:\.babel)?|runt(?:file)?)\.js|b(?:inding\.gyp|ench.*\.js|ower\.json)|.*\.(?:m(?:arkdown|d)|pdf|txt)|r(?:ollup\.config\.js|eadme)|t(?:sconfig\.json|ests?\.js)|l(?:icen[cs]e.*|erna\.json)|(?:Docker|make)file|package-lock\.json|yarn\.lock|history|\..*)$/i;
+my $filesIgnored =
+qr/^(?:\.(?:git(?:ignore)?|npmignore|eslint.*)|.*\.(?:c(?:offee|\+\+|cp?)?|def|mk|h)|(?:binding.M|m)akefile)$/i;
 
 # Regexp compiled from dh_nodejs/dh_dirs.excluded
 my $dirIgnored =
-  qr{^(?:(?:example|test|doc)s?|node_modules|bench|\..*)(?:/.*)?}i;
+qr{^(?:(?:node_module|(.*/)?.dep)s|(?:example|test|doc)s?|bench|\..*)(?:/.*)?}i;
 
-# Extensions that overrides $fileIgnored if set in package.json#files
+# Extensions that overrides $rootFilesIgnored if set in package.json#files
 my $authorizedExt = qr/\.(?:js)$/;
 
 ### PRIVATE METHODS
@@ -95,7 +97,12 @@ sub pjson {
     open $pkgjson, "$dir/package.json";
     my $content = join '', <$pkgjson>;
     close $pkgjson;
-    return $json{$dir} = JSON::from_json($content);
+    $json{$dir} = JSON::from_json($content);
+    if ( -e "debian/nodejs/$dir/name" ) {
+        $json{$dir}->{name} =
+          [ $self->open_file("debian/nodejs/$dir/name") ]->[0];
+    }
+    return $json{$dir};
 }
 
 sub component_list {
@@ -106,20 +113,24 @@ sub component_list {
           $self->open_file('debian/watch');
     }
     if ( -e 'debian/nodejs/additional_components' ) {
-        map { push @components, glob($_) }
-          $self->open_file('debian/nodejs/additional_components');
+        print "Found debian/nodejs/additional_components\n";
+        map {
+            print 'Adding component(s): ' . join( ', ', glob($_) ) . "\n";
+            push @components, glob($_)
+        } $self->open_file('debian/nodejs/additional_components');
     }
     foreach my $component (@components) {
         next if $self->main_package and $component eq $self->main_package;
         if ( -d $component ) {
             my $package;
             eval { $package = $self->pjson($component)->{name} };
-            print STDERR $@ if $@;
+            print STDERR "Unable to load $component: $@" if $@;
             next if ($@);
             $packages{$component} = $package;
         }
         else {
-            die "Can't find $component directory in " . `pwd`;
+            print STDERR "Can't find $component directory in " . `pwd`;
+            next;
         }
     }
     return \%packages;
@@ -333,12 +344,19 @@ sub install_module {
         $noFilesField = 0;
         $skipFiles    = 0;
     }
-    elsif ( $self->pjson($dir)->{files} or $self->pjson($dir)->{types} ) {
+    elsif ( $self->pjson($dir)->{files} ) {
         @files = $self->readFilesField(
             $dir,
             $self->filesFieldToList( $dir, 'files' ),
-            $self->filesFieldToList( $dir, 'types' )
+            $self->filesFieldToList( $dir, 'typings' ),
+            $self->filesFieldToList( $dir, 'types' ),
         );
+        if ( $self->pjson($dir)->{types} and not $self->pjson($dir)->{typings} )
+        {
+            print STDERR
+              qq{# /!\ "types" field should be replaced by "typings"}
+              . " in $dir/package.json\nPlease report this bug\n";
+        }
         $noFilesField = 0;
     }
     else {
@@ -388,16 +406,25 @@ sub install_module {
                     ( $d and $d =~ $re and $skipFiles ),
 
                     # Ignore our debian/ dir
-                    ( $File::Find::dir =~ m{^(?:./)?debian} and $skipFiles ),
+                    (
+                              $dir eq '.'
+                          and $File::Find::dir =~ m{^(?:./)?debian}
+                          and $skipFiles
+                    ),
 
                     # Ignore license, changelog, readme,... files
-                    ( $_ =~ $fileIgnored and $noFilesField and !$d ),
+                    ( $_ =~ $rootFilesIgnored and $noFilesField and !$d ),
                     (
-                              $_ =~ $fileIgnored
+                              $_ =~ $rootFilesIgnored
+                          and $skipFiles
                           and ( $noFilesField or $_ !~ $authorizedExt )
                           and !$d
                     ),
 
+          # Ignore Makefile, .eslint anywhere unless DM specified it explicitely
+                    ( $_ =~ $filesIgnored and ( $pattern or $p !~ /$_$/ ) ),
+                    ( $_ =~ $filesIgnored and ( $pattern or $p !~ /$_$/ ) ),
+
                   # Ignore test directory unless specified in package.json#files
                     ( $d and $d =~ $dirIgnored and $noFilesField ),
 
@@ -407,9 +434,6 @@ sub install_module {
                           and /^(?:example|test)s?\.js$/i
                           and $noFilesField
                     ),
-
-                    # Exclude any .eslint* unless DM specified it explicitely
-                    ( /^\.eslint.*/ and ( $pattern or $p !~ /$_$/ ) ),
                 );
 
                 # Install files unless one exclusion test match
@@ -447,12 +471,12 @@ EOF
             $p
         );
     }
-    unless ( $foundMain and -e $mainFile ) {
-        print STDERR
-          "package.json#main file not installed, forcing it ($mainFile)\n";
-        my $dir = '';
-        $dir = $1 if $mainFile =~ m#^(.*)/#;
-        push @dest, [ $dir, $mainFile ] if -e $mainFile;
+    unless ( $foundMain or !$self->main_package ) {
+        if ( -e $mainFile ) {
+            print STDERR
+              "package.json#main file not installed, forcing it ($mainFile)\n";
+            push @dest, [ "", $mainFile ];
+        }
     }
     foreach (@dest) {
         $self->doit_in_builddir( 'mkdir', '-p', "$path/$_->[0]" )
@@ -468,8 +492,10 @@ sub filesFieldToList {
     my ( $self, $dir, $field ) = @_;
     return () unless $self->pjson($dir)->{$field};
     print qq{Found "$field" field in $dir/package.json, using it\n};
-    return @{ $self->pjson($dir)->{$field} }
-      if ref $self->pjson($dir)->{$field};
+    if ( my $ref = ref $self->pjson($dir)->{$field} ) {
+        return ( values %{ $self->pjson($dir)->{$field} } ) if $ref eq 'HASH';
+        return @{ $self->pjson($dir)->{$field} };
+    }
     return ( "$dir/" . $self->pjson($dir)->{$field} );
 }
 
@@ -477,6 +503,7 @@ sub readFilesField {
     my ( $self, $dir, @lines ) = @_;
     my @files = map {
         $_ = $self->toJs( $dir, $_ );
+        s#^\.?/##;
         print STDERR "$dir/package.json#files: $_ does not exists\n"
           unless $_ and -e "$dir/$_" or /[\{\*]/;
         $_ ? "$dir/$_" : ();
@@ -489,7 +516,10 @@ sub readFilesField {
 sub toJs {
     my ( $self, $dir, $file ) = @_;
     return (
-        ( -e "$dir/$file.js" and not -e "$dir/$file" ) ? "$file.js" : "$file" );
+        ( -e "$dir/$file.js" and not -e "$dir/$file" )
+        ? "$file.js"
+        : "$file"
+    );
 }
 
 sub cmp_ordered_list {
@@ -545,8 +575,9 @@ sub configure {
     my $components = $self->component_list;
     my @dirs       = ( sort keys %$components );
     return
-      unless @dirs
-      or -e 'debian/nodejs/' . $self->main_package . '/extlinks';
+         unless @dirs
+      or -e 'debian/nodejs/' . $self->main_package . '/extlinks'
+      or -e 'debian/nodejs/' . $self->main_package . '/extcopies';
     $self->doit_in_builddir( 'mkdir', 'node_modules' )
       unless -e 'node_modules';
 
@@ -556,9 +587,30 @@ sub configure {
             map {
                 my $src = `nodepath $_`;
                 chomp $src;
-                $self->dolink( $src, "$dir/node_modules/" );
+                my $tmp = $src;
+                $tmp =~ s#^.*?nodejs/##;
+                $tmp =~ s#/.*?$## or $tmp = '';
+                $tmp = "$tmp/" if $tmp;
+                $self->dolink( $src, "$dir/node_modules/$tmp" );
             } $self->open_file("debian/nodejs/$dir/extlinks");
         }
+        if ( -e "debian/nodejs/$dir/extcopies" ) {
+            map {
+                my $src = `nodepath $_`;
+                chomp $src;
+                my $dst = $src;
+                $dst =~ s#.*?nodejs/##;
+                if ( $dst =~ s#/[^/]*$## ) {
+                    $self->doit_in_builddir( 'mkdir', '-p',
+                        "$dir/node_modules/$dst" );
+                }
+                else {
+                    $dst = '';
+                }
+                $self->doit_in_builddir( 'cp', '-r', $src,
+                    "$dir/node_modules/$dst" );
+            } $self->open_file("debian/nodejs/$dir/extcopies");
+        }
     }
 
     # Link each component in node_modules
@@ -616,8 +668,12 @@ sub test {
                 ( '../' x ( scalar(@count) + 1 ) ) . "debian/nodejs/$_/test" );
         }
     }
-    if ( -e 'debian/tests/pkg-js/test' ) {
-        $self->doit_in_builddir( '/bin/sh', '-e', 'debian/tests/pkg-js/test' );
+    my $testfile =
+      ( -e 'debian/nodejs/test'
+        ? 'debian/nodejs/test'
+        : 'debian/tests/pkg-js/test' );
+    if ( -e $testfile ) {
+        $self->doit_in_builddir( '/bin/sh', '-e', $testfile );
     }
     else {
         $self->doit_in_builddir( '/usr/bin/node', '-e',
@@ -641,12 +697,23 @@ sub install {
         package_is_arch_all($package)
         ? '/usr/share/nodejs'
         : '/usr/lib/'
-          . dpkg_architecture_value("DEB_HOST_GNU_TYPE")
+          . dpkg_architecture_value("DEB_HOST_MULTIARCH")
           . '/nodejs'
     );
     my $archPath = "$destdir/$realArchPath";
     my $node_module =
-      $self->main_package ? $self->pjson( $self->main_package )->{name} : '';
+        $self->main_package
+      ? $self->pjson( $self->main_package )->{name}
+      : '';
+
+    unless ($node_module) {
+        print STDERR <<EOT;
+##############################################################
+# /!\ No name found for main module, this may break install  #
+# To fix this, fix main component path in debian/nodejs/main #
+##############################################################
+EOT
+    }
     my $components = $self->component_list;
     my @dirs       = ( sort keys %$components );
 
@@ -656,23 +723,8 @@ sub install {
         @dirs )
       if $node_module;
 
-    # Links
-    if ( -e "debian/nodejs/links" ) {
-        my $tmp = tmpdir($package);
-        map {
-            my $line = $_;
-            my ( $src, $dest ) =
-              map { m#^/# ? $_ : "$realArchPath/$_" } split /\s+/, $line;
-            unless ( $src and $dest ) {
-                die "Bad line in debian/nodejs/links: $_";
-            }
-            print "Linking $dest to $src\n";
-            make_symlink( $dest, $src, $tmp );
-        } $self->open_file("debian/nodejs/links");
-    }
-
     # Component install
-    return unless (@dirs);
+    #return unless (@dirs);
 
     # Search root components
     my %root_cmp;
@@ -692,12 +744,12 @@ sub install {
         %root_cmp = map {
             my $s = $_;
             map { ( $_ => 1 ) } glob($s)
-        } $self->open_file('debian/nodejs/root_modules');
+        } @dirs;
     }
 
     $path = "$path/node_modules";
     if ( -e 'debian/nodejs/submodules' ) {
-        my @d = map { chomp; /^\w/ ? $_ : () }
+        my @d = map { chomp; /^\w/ ? ( glob($_) ) : () }
           $self->open_file('debian/nodejs/submodules');
         unless (@d) {
             print
@@ -706,16 +758,20 @@ sub install {
         }
         my @tmp;
       CMP: foreach my $cmp (@d) {
+            next CMP
+              if $self->main_package and $cmp eq $self->main_package;
             foreach my $dir (@dirs) {
                 if ( $dir eq $cmp or $components->{$dir} eq $cmp ) {
                     push @tmp, $dir;
                     next CMP;
                 }
             }
-            die qq{In debian/nodejs/submodules, "$cmp" matches nothing};
+            print STDERR
+              qq{# /!\ In debian/nodejs/submodules, "$cmp" matches nothing\n};
         }
         @dirs = @tmp;
     }
+
     my @provides;
     foreach my $dir (@dirs) {
         $self->install_module(
@@ -731,7 +787,8 @@ sub install {
         );
         if ( $root_cmp{$dir} ) {
             my $tmp = 'node-' . $self->pjson($dir)->{name};
-            $tmp =~ s/_/-/g;
+            $tmp =~ s/[\@_\/]/-/g;
+            $tmp =~ s/--*/-/g;
             if ( my $v = $self->pjson($dir)->{version} ) {
                 $tmp .= " (= $v)";
             }
@@ -740,6 +797,22 @@ sub install {
     }
     addsubstvar( $package, 'nodejs:Provides', join( ', ', @provides ) )
       if @provides;
+
+    # Links
+    if ( -e "debian/nodejs/links" ) {
+        my $tmp = tmpdir($package);
+        map {
+            my $line = $_;
+            my ( $src, $dest ) =
+              map { m#^/# ? $_ : "$realArchPath/$_" } split /\s+/, $line;
+            unless ( $src and $dest ) {
+                die "Bad line in debian/nodejs/links: $_";
+            }
+            print "Linking $dest to $src\n";
+            make_symlink( $dest, $src, $tmp );
+        } $self->open_file("debian/nodejs/links");
+    }
+
 }
 
 # clean step: try to clean node_modules directories
@@ -753,6 +826,11 @@ sub clean {
             map { $self->do_unlink("$dir/node_modules/$_"); }
               $self->open_file("debian/nodejs/$dir/extlinks");
         }
+        if ( -e "debian/nodejs/$dir/extcopies" ) {
+            map {
+                $self->doit_in_builddir( 'rm', '-rf', "$dir/node_modules/$_" );
+            } $self->open_file("debian/nodejs/$dir/extcopies");
+        }
     }
     if ( -e 'debian/nodejs/component_links' ) {
         map {
