@@ -5,10 +5,13 @@ package Lintian::pkg_js::repo;
 use strict;
 use warnings;
 use Dpkg::IPC;
-use Lintian::Tags qw(tag);
+use Moo;
+use namespace::clean;
 
-sub run {
-    my ( $pkg, $type, $info, $proc, $group ) = @_;
+with 'Lintian::Check';
+
+sub source {
+    my ( $self ) = @_;
     my ( $out, $err );
     return unless ( -e 'package.json' );
     spawn(
@@ -18,7 +21,7 @@ sub run {
         wait_child      => 1,
         nocheck         => 1
     );
-    tag 'inconsistency-debian-watch' if $?;
+    $self->tag('inconsistency-debian-watch') if $?;
 }
 
 1;
