@@ -26,33 +26,38 @@ package Lintian::pkg_js::vcs;
 use strict;
 use warnings;
 
-use Lintian::Tags qw(tag);
+use Moo;
+use namespace::clean;
 
-sub run {
-    my ( $pkg, $type, $info, $proc, $group ) = @_;
+with 'Lintian::Check';
+
+sub source {
+    my ( $self ) = @_;
+
+    my $processable = $self->processable;
 
     # Only for pkg-js packages
     return
-      unless $info->field('maintainer') =~
+      unless $processable->field('maintainer') =~
       /pkg-javascript-maintainers\@lists\.alioth\.debian\.org/sm;
 
     # Check if the Vcs headers are git
     my @non_git_vcs_fields =
       qw(vcs-arch vcs-bzr vcs-cvs vcs-darcs vcs-hg vcs-mtn vcs-svn);
     foreach my $field_name (@non_git_vcs_fields) {
-        my $field_value = $info->field($field_name);
+        my $field_value = $processable->field($field_name);
         if ( defined($field_value) ) {
-            tag 'pkg-js-vcs-not-git', "$field_name";
+            $self->tag('pkg-js-vcs-not-git', "$field_name");
         }
     }
 
     # Check if repo is writable for the Debian Perl Group
     foreach my $field_name ( @non_git_vcs_fields, qw(vcs-git vcs-browser) ) {
-        my $field_value = $info->field($field_name);
+        my $field_value = $processable->field($field_name);
         if ( defined($field_value)
             && $field_value !~ m{^https://salsa.debian.org/js-team}i )
         {
-            tag 'vcs-url-not-under-js-team', "$field_value";
+            $self->tag('vcs-url-not-under-js-team', "$field_value");
         }
     }
     return;
